<?php

namespace Drupal\Tests\interactive_queue_processing\Functional;

use Drupal\interactive_queue_processing_test\Plugin\QueueWorker\NodeCreateQueueWorker;
use Drupal\node\NodeInterface;

/**
 * Functional tests.
 */
class InteractiveQueueProcessingBrowserTest extends InteractiveQueueProcessingBrowserTestBase {

  /**
   * Tests that the AJAX processing endpoint is secured by permission.
   */
  public function testProcessQueuePermission() {
    // Anonymous.
    $this->drupalGet('interactive-queue-processing/ajax');
    $this->assertSession()->statusCodeEquals(403);

    // Authenticated w/permission.
    $account = $this->drupalCreateUser(['interactively process queues']);
    $this->drupalLogin($account);
    $this->drupalGet('interactive-queue-processing/ajax');
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests that the max items configuration is respected.
   */
  public function testMaxItems() {
    $this->drupalCreateContentType(['type' => NodeCreateQueueWorker::TYPE]);

    $queue = \Drupal::queue(NodeCreateQueueWorker::ID);
    $queue->createItem(['title' => 1]);
    $queue->createItem(['title' => 2]);
    $queue->createItem(['title' => 3]);
    $queue->createItem(['title' => 4]);
    $queue->createItem(['title' => 5]);
    $queue->createItem(['title' => 6]);
    $queue->createItem(['title' => 7]);
    $queue->createItem(['title' => 8]);
    $queue->createItem(['title' => 9]);
    $queue->createItem(['title' => 10]);
    $queue->createItem(['title' => 11]);
    $queue->createItem(['title' => 12]);
    $queue->createItem(['title' => 13]);
    $queue->createItem(['title' => 14]);
    $queue->createItem(['title' => 15]);
    $queue->createItem(['title' => 16]);
    $queue->createItem(['title' => 17]);
    $queue->createItem(['title' => 18]);
    $queue->createItem(['title' => 19]);
    $queue->createItem(['title' => 20]);

    // Limit 3.
    $config = $this->config('interactive_queue_processing.settings');
    $config->set('max_items', 3);
    $config->save();

    $this->allowAnonProcessing();
    $this->processQueues();

    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(1));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(2));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(3));
    $this->assertFalse($this->drupalGetNodeByTitle(4));
    $this->assertFalse($this->drupalGetNodeByTitle(5));
    $this->assertFalse($this->drupalGetNodeByTitle(6));
    $this->assertFalse($this->drupalGetNodeByTitle(7));
    $this->assertFalse($this->drupalGetNodeByTitle(8));
    $this->assertFalse($this->drupalGetNodeByTitle(9));
    $this->assertFalse($this->drupalGetNodeByTitle(10));
    $this->assertFalse($this->drupalGetNodeByTitle(11));
    $this->assertFalse($this->drupalGetNodeByTitle(12));
    $this->assertFalse($this->drupalGetNodeByTitle(13));
    $this->assertFalse($this->drupalGetNodeByTitle(14));
    $this->assertFalse($this->drupalGetNodeByTitle(15));
    $this->assertFalse($this->drupalGetNodeByTitle(16));
    $this->assertFalse($this->drupalGetNodeByTitle(17));
    $this->assertFalse($this->drupalGetNodeByTitle(18));
    $this->assertFalse($this->drupalGetNodeByTitle(19));
    $this->assertFalse($this->drupalGetNodeByTitle(20));

    // Limit 5.
    $config->set('max_items', 5);
    $config->save();

    $this->allowAnonProcessing();
    $this->processQueues();

    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(1));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(2));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(3));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(4));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(5));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(6));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(7));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(8));
    $this->assertFalse($this->drupalGetNodeByTitle(9));
    $this->assertFalse($this->drupalGetNodeByTitle(10));
    $this->assertFalse($this->drupalGetNodeByTitle(11));
    $this->assertFalse($this->drupalGetNodeByTitle(12));
    $this->assertFalse($this->drupalGetNodeByTitle(13));
    $this->assertFalse($this->drupalGetNodeByTitle(14));
    $this->assertFalse($this->drupalGetNodeByTitle(15));
    $this->assertFalse($this->drupalGetNodeByTitle(16));
    $this->assertFalse($this->drupalGetNodeByTitle(17));
    $this->assertFalse($this->drupalGetNodeByTitle(18));
    $this->assertFalse($this->drupalGetNodeByTitle(19));
    $this->assertFalse($this->drupalGetNodeByTitle(20));

    // Unlimited.
    $config->set('max_items', -1);
    $config->save();

    $this->allowAnonProcessing();
    $this->processQueues();

    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(1));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(2));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(3));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(4));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(5));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(6));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(7));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(8));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(9));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(10));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(11));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(12));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(13));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(14));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(15));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(16));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(17));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(18));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(19));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(20));
  }

  public function testProcessingTime() {
    $queue = \Drupal::queue(NodeCreateQueueWorker::ID);
    $queue->createItem(['title' => 1, 'sleep' => 1000]);
    $queue->createItem(['title' => 2, 'sleep' => 1000]);
    $queue->createItem(['title' => 3, 'sleep' => 1000]);
    $queue->createItem(['title' => 4, 'sleep' => 1000]);
    $queue->createItem(['title' => 5, 'sleep' => 1000]);
    $queue->createItem(['title' => 6, 'sleep' => 1000]);
    $queue->createItem(['title' => 7, 'sleep' => 1000]);
    $queue->createItem(['title' => 8, 'sleep' => 1000]);
    $queue->createItem(['title' => 9, 'sleep' => 1000]);
    $queue->createItem(['title' => 10, 'sleep' => 1000]);
    $queue->createItem(['title' => 11, 'sleep' => 1000]);
    $queue->createItem(['title' => 12, 'sleep' => 1000]);
    $queue->createItem(['title' => 13, 'sleep' => 1000]);
    $queue->createItem(['title' => 14, 'sleep' => 1000]);
    $queue->createItem(['title' => 15, 'sleep' => 1000]);
    $queue->createItem(['title' => 16, 'sleep' => 1000]);
    $queue->createItem(['title' => 17, 'sleep' => 1000]);
    $queue->createItem(['title' => 18, 'sleep' => 1000]);
    $queue->createItem(['title' => 19, 'sleep' => 1000]);
    $queue->createItem(['title' => 20, 'sleep' => 1000]);

    // Limit 3.
    $config = $this->config('interactive_queue_processing.settings');
    $config->set('processing_time', 3);
    $config->save();

    $this->allowAnonProcessing();
    $this->processQueues();

    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(1));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(2));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(3));
    $this->assertFalse($this->drupalGetNodeByTitle(4));
    $this->assertFalse($this->drupalGetNodeByTitle(5));
    $this->assertFalse($this->drupalGetNodeByTitle(6));
    $this->assertFalse($this->drupalGetNodeByTitle(7));
    $this->assertFalse($this->drupalGetNodeByTitle(8));
    $this->assertFalse($this->drupalGetNodeByTitle(9));
    $this->assertFalse($this->drupalGetNodeByTitle(10));
    $this->assertFalse($this->drupalGetNodeByTitle(11));
    $this->assertFalse($this->drupalGetNodeByTitle(12));
    $this->assertFalse($this->drupalGetNodeByTitle(13));
    $this->assertFalse($this->drupalGetNodeByTitle(14));
    $this->assertFalse($this->drupalGetNodeByTitle(15));
    $this->assertFalse($this->drupalGetNodeByTitle(16));
    $this->assertFalse($this->drupalGetNodeByTitle(17));
    $this->assertFalse($this->drupalGetNodeByTitle(18));
    $this->assertFalse($this->drupalGetNodeByTitle(19));
    $this->assertFalse($this->drupalGetNodeByTitle(20));

    // Limit 5.
    $config->set('processing_time', 5);
    $config->save();

    $this->allowAnonProcessing();
    $this->processQueues();

    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(1));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(2));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(3));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(4));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(5));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(6));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(7));
    $this->assertInstanceOf(NodeInterface::class, $this->drupalGetNodeByTitle(8));
    $this->assertFalse($this->drupalGetNodeByTitle(9));
    $this->assertFalse($this->drupalGetNodeByTitle(10));
    $this->assertFalse($this->drupalGetNodeByTitle(11));
    $this->assertFalse($this->drupalGetNodeByTitle(12));
    $this->assertFalse($this->drupalGetNodeByTitle(13));
    $this->assertFalse($this->drupalGetNodeByTitle(14));
    $this->assertFalse($this->drupalGetNodeByTitle(15));
    $this->assertFalse($this->drupalGetNodeByTitle(16));
    $this->assertFalse($this->drupalGetNodeByTitle(17));
    $this->assertFalse($this->drupalGetNodeByTitle(18));
    $this->assertFalse($this->drupalGetNodeByTitle(19));
    $this->assertFalse($this->drupalGetNodeByTitle(20));
  }

}
